# Copyright (c) 2020 Mike Kinney
"""Tests for VMrun class."""
from unittest.mock import patch, MagicMock, mock_open


import mech.vmrun


@patch('subprocess.Popen')
def test_vmrun_vmrun(mock_popen):
    """Test vmrun method."""
    process_mock = MagicMock()
    attrs = {'communicate.return_value': ('output', 'error')}
    process_mock.configure_mock(**attrs)
    mock_popen.return_value = process_mock
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx',
                             executable='/tmp/vmrun', provider='ws')
    got = vmrun.vmrun('list', vmrun.vmx_file)
    assert got is None
    mock_popen.assert_called()


@patch('os.path.exists', return_value=True)
def test_vmrun_installed(mock_path_exists):
    """Test installed."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/bin/vmrun',
                             user='admin', password='1234', provider='ws', test_mode=True)
    assert vmrun.installed()
    mock_path_exists.assert_called()


@patch('os.path.exists', return_value=False)
def test_vmrun_executable_not_found(mock_path_exists):
    """Test installed."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/bin/vmrun',
                             user='admin', password='1234', provider='ws', test_mode=True)
    assert not vmrun.installed()
    mock_path_exists.assert_called()


def test_vmrun_start():
    """Test start method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             user='admin', password='1234', provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', '-gu', 'admin', '-gp', '1234',
                'start', '/tmp/first/some.vmx', 'nogui']
    got = vmrun.start()
    assert got == expected


def test_vmrun_stop():
    """Test stop method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'stop', '/tmp/first/some.vmx', 'soft']
    got = vmrun.stop()
    assert got == expected


def test_vmrun_reset():
    """Test reset method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'reset', '/tmp/first/some.vmx', 'soft']
    got = vmrun.reset()
    assert got == expected


def test_vmrun_suspend():
    """Test suspend method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'suspend', '/tmp/first/some.vmx', 'soft']
    got = vmrun.suspend()
    assert got == expected


def test_vmrun_pause():
    """Test pause method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'pause', '/tmp/first/some.vmx']
    got = vmrun.pause()
    assert got == expected


def test_vmrun_unpause():
    """Test unpause method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'unpause', '/tmp/first/some.vmx']
    got = vmrun.unpause()
    assert got == expected


def test_vmrun_list_snapshots():
    """Test list_snapshots method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'listSnapshots', '/tmp/first/some.vmx']
    got = vmrun.list_snapshots()
    assert got == expected


def test_vmrun_snapshot():
    """Test snapshot method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'snapshot', '/tmp/first/some.vmx', 'snap1']
    got = vmrun.snapshot('snap1')
    assert got == expected


def test_vmrun_delete_snapshot():
    """Test delete_snapshot method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'deleteSnapshot', '/tmp/first/some.vmx', 'snap1']
    got = vmrun.delete_snapshot('snap1')
    assert got == expected


def test_vmrun_revert_to_snapshot():
    """Test revert_to_snapshot method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'revertToSnapshot', '/tmp/first/some.vmx', 'snap2']
    got = vmrun.revert_to_snapshot('snap2')
    assert got == expected


def test_vmrun_list_network_adapters():
    """Test list_network_adapters method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'listNetworkAdapters', '/tmp/first/some.vmx']
    got = vmrun.list_network_adapters('snap2')
    assert got == expected


def test_vmrun_add_network_adapter():
    """Test add_network_adapter method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'addNetworkAdapter', '/tmp/first/some.vmx', 'a_type']
    got = vmrun.add_network_adapter('a_type')
    assert got == expected


def test_vmrun_set_network_adapter():
    """Test set_network_adapter method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'setNetworkAdapter',
                '/tmp/first/some.vmx', 'some_index', 'a_type']
    got = vmrun.set_network_adapter('some_index', 'a_type')
    assert got == expected


def test_vmrun_delete_network_adapter():
    """Test delete_network_adapter method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'deleteNetworkAdapter', '/tmp/first/some.vmx', 'a_type']
    got = vmrun.delete_network_adapter('a_type')
    assert got == expected


def test_vmrun_list_networks():
    """Test list_host_networks method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'listHostNetworks']
    got = vmrun.list_host_networks()
    assert got == expected


def test_vmrun_list_port_forwardings():
    """Test list_port_forwardings method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'listPortForwardings', 'a_host_network']
    got = vmrun.list_port_forwardings('a_host_network')
    assert got == expected


def test_vmrun_set_port_forwarding():
    """Test set_port_forwarding method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'setPortForwarding',
                'a_host_network', 'a_protocol', 'a_host_port', 'a_guest_ip', 'a_guest_port']
    got = vmrun.set_port_forwarding('a_host_network', 'a_protocol',
                                    'a_host_port', 'a_guest_ip', 'a_guest_port')
    assert got == expected


def test_vmrun_delete_port_forwarding():
    """Test delete_port_forwarding method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'deletePortForwarding',
                'a_host_network', 'a_protocol', 'a_host_port']
    got = vmrun.delete_port_forwarding('a_host_network', 'a_protocol', 'a_host_port')
    assert got == expected


def test_vmrun_run_program_in_guest():
    """Test run_program_in_guest method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'runProgramInGuest', '/tmp/first/some.vmx',
                'program_path', 'one_cmd']
    got = vmrun.run_program_in_guest('program_path', ['one_cmd'])
    assert got == expected


def test_vmrun_run_program_in_guest_no_args():
    """Test run_program_in_guest method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'runProgramInGuest', '/tmp/first/some.vmx',
                'program_path']
    got = vmrun.run_program_in_guest('program_path')
    assert got == expected


def test_vmrun_run_program_in_guest_non_defaults():
    """Test run_program_in_guest method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'runProgramInGuest', '/tmp/first/some.vmx',
                '-noWait', '-activateWindow', '-interactive', 'program_path', 'one_cmd']
    got = vmrun.run_program_in_guest('program_path', ['one_cmd'], wait=False,
                                     activate_window=True, interactive=True)
    assert got == expected


def test_vmrun_set_shared_folder_state():
    """Test set_shared_folder_state method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'setSharedFolderState', '/tmp/first/some.vmx',
                'a_share_name', 'a_new_path', 'readonly']
    got = vmrun.set_shared_folder_state('a_share_name', 'a_new_path')
    assert got == expected


def test_vmrun_add_shared_folder():
    """Test add_shared_folder method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'addSharedFolder', '/tmp/first/some.vmx',
                'a_share_name', 'a_path']
    got = vmrun.add_shared_folder('a_share_name', 'a_path')
    assert got == expected


def test_vmrun_remove_shared_folder():
    """Test remove_shared_folder method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'removeSharedFolder', '/tmp/first/some.vmx',
                'a_share_name']
    got = vmrun.remove_shared_folder('a_share_name')
    assert got == expected


def test_vmrun_enable_shared_folders():
    """Test enable_shared_folders method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'enableSharedFolders', '/tmp/first/some.vmx']
    got = vmrun.enable_shared_folders()
    assert got == expected


def test_vmrun_disable_shared_folders():
    """Test disable_shared_folders method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'disableSharedFolders', '/tmp/first/some.vmx']
    got = vmrun.disable_shared_folders()
    assert got == expected


def test_vmrun_list_processes_in_guest():
    """Test list_processes_in_guest method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'listProcessesInGuest', '/tmp/first/some.vmx']
    got = vmrun.list_processes_in_guest()
    assert got == expected


def test_vmrun_kill_process_in_guest():
    """Test kill_process_in_guest method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'killProcessInGuest', '/tmp/first/some.vmx', 'a_pid']
    got = vmrun.kill_process_in_guest('a_pid')
    assert got == expected


def test_vmrun_run_script_in_guest():
    """Test run_script_in_guest method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'runScriptInGuest', '/tmp/first/some.vmx',
                'a_path', 'a_script']
    got = vmrun.run_script_in_guest('a_path', 'a_script')
    assert got == expected


def test_vmrun_delete_file_in_guest():
    """Test delete_file_in_guest method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'deleteFileInGuest', '/tmp/first/some.vmx', 'a_file']
    got = vmrun.delete_file_in_guest('a_file')
    assert got == expected


def test_vmrun_create_directory_in_guest():
    """Test create_directory_in_guest method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'createDirectoryInGuest', '/tmp/first/some.vmx', 'a_dir']
    got = vmrun.create_directory_in_guest('a_dir')
    assert got == expected


def test_vmrun_delete_directory_in_guest():
    """Test delete_directory_in_guest method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'deleteDirectoryInGuest', '/tmp/first/some.vmx', 'a_dir']
    got = vmrun.delete_directory_in_guest('a_dir')
    assert got == expected


def test_vmrun_create_tempfile_in_guest():
    """Test create_tempfile_in_guest method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'createTempfileInGuest', '/tmp/first/some.vmx']
    got = vmrun.create_tempfile_in_guest()
    assert got == expected


def test_vmrun_list_directory_in_guest():
    """Test list_directory_in_guest method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'listDirectoryInGuest', '/tmp/first/some.vmx', 'a_path']
    got = vmrun.list_directory_in_guest('a_path')
    assert got == expected


def test_vmrun_copy_file_from_host_to_guest():
    """Test copy_file_from_host_to_guest method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'copyFileFromHostToGuest', '/tmp/first/some.vmx',
                'a_host_path', 'a_guest_path']
    got = vmrun.copy_file_from_host_to_guest('a_host_path', 'a_guest_path')
    assert got == expected


def test_vmrun_copy_file_from_guest_to_host():
    """Test copy_file_from_guest_to_host method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'copyFileFromGuestToHost', '/tmp/first/some.vmx',
                'a_guest_path', 'a_host_path']
    got = vmrun.copy_file_from_guest_to_host('a_guest_path', 'a_host_path')
    assert got == expected


def test_vmrun_rename_file_in_guest():
    """Test rename_file_in_guest method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'renameFileInGuest', '/tmp/first/some.vmx',
                'orig', 'new']
    got = vmrun.rename_file_in_guest('orig', 'new')
    assert got == expected


def test_vmrun_type_keystrokes_in_guest():
    """Test type_keystrokes_in_guest method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'typeKeystrokesInGuest', '/tmp/first/some.vmx',
                'some_keys']
    got = vmrun.type_keystrokes_in_guest('some_keys')
    assert got == expected


def test_vmrun_connect_named_device():
    """Test connect_named_device method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'connectNamedDevice', '/tmp/first/some.vmx',
                'a_device_name']
    got = vmrun.connect_named_device('a_device_name')
    assert got == expected


def test_vmrun_disconnect_named_device():
    """Test disconnect_named_device method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'disconnectNamedDevice', '/tmp/first/some.vmx',
                'a_device_name']
    got = vmrun.disconnect_named_device('a_device_name')
    assert got == expected


def test_vmrun_capture_screen():
    """Test capture_screen method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'captureScreen', '/tmp/first/some.vmx',
                'a_path_on_host']
    got = vmrun.capture_screen('a_path_on_host')
    assert got == expected


def test_vmrun_write_variable():
    """Test write_variable method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'writeVariable', '/tmp/first/some.vmx',
                'a_name', 'a_value']
    got = vmrun.write_variable('a_name', 'a_value')
    assert got == expected


def test_vmrun_read_variable():
    """Test read_variable method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'readVariable', '/tmp/first/some.vmx', 'a_name']
    got = vmrun.read_variable('a_name')
    assert got == expected


@patch('mech.vmrun.VMrun.vmrun', return_value='192.168.1.200')
def test_vmrun_get_guest_ip_address_no_lookup(mock_vmrun):
    """Test get_guest_ip_address method without lookup."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx',
                             executable='/tmp/vmrun', provider='ws')
    got = vmrun.get_guest_ip_address()
    assert got == '192.168.1.200'
    mock_vmrun.assert_called()


@patch('mech.vmrun.VMrun.vmrun', return_value='unknown')
def test_vmrun_get_guest_ip_address_no_lookup_unknown(mock_vmrun):
    """Test get_guest_ip_address method without lookup."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx',
                             executable='/tmp/vmrun', provider='ws')
    got = vmrun.get_guest_ip_address()
    assert got == ''
    mock_vmrun.assert_called()


@patch('mech.vmrun.VMrun.copy_file_from_guest_to_host', return_value='')
@patch('mech.vmrun.VMrun.run_script_in_guest', return_value='')
def test_vmrun_get_guest_ip_address_lookup(mock_run_script_in_guest,
                                           mock_copy_file_from_guest_to_host):
    """Test get_guest_ip_address method with lookup."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', user='auser', password='apass',
                             executable='/tmp/vmrun', provider='ws')
    a_mock = mock_open()
    a_mock = mock_open(read_data='192.168.1.201')
    with patch('builtins.open', a_mock, create=True):
        got = vmrun.get_guest_ip_address(lookup=True)
        assert got == '192.168.1.201'
    a_mock.assert_called()
    mock_run_script_in_guest.assert_called()
    mock_copy_file_from_guest_to_host.assert_called()


def test_vmrun_list():
    """Test list method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'list', '/tmp/first/some.vmx']
    got = vmrun.list()
    assert got == expected


def test_vmrun_upgradevm():
    """Test upgradevm method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'upgradevm', '/tmp/first/some.vmx']
    got = vmrun.upgradevm()
    assert got == expected


def test_vmrun_install_tools():
    """Test install_tools method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'installTools', '/tmp/first/some.vmx']
    got = vmrun.install_tools()
    assert got == expected


def test_vmrun_check_tools_state():
    """Test check_tools_state method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'checkToolsState', '/tmp/first/some.vmx']
    got = vmrun.check_tools_state()
    assert got == expected


def test_vmrun_register():
    """Test register method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'register', '/tmp/first/some.vmx']
    got = vmrun.register()
    assert got == expected


def test_vmrun_unregister():
    """Test unregister method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'unregister', '/tmp/first/some.vmx']
    got = vmrun.unregister()
    assert got == expected


def test_vmrun_list_registered_vm():
    """Test list_registered_vm method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'listRegisteredVM', '/tmp/first/some.vmx']
    got = vmrun.list_registered_vm()
    assert got == expected


def test_vmrun_delete_vm():
    """Test delete_vm method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'deleteVM', '/tmp/first/some.vmx']
    got = vmrun.delete_vm()
    assert got == expected


def test_vmrun_clone():
    """Test clone method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'clone', '/tmp/first/some.vmx',
                'a_dest_vmx', 'a_mode']
    got = vmrun.clone('a_dest_vmx', 'a_mode')
    assert got == expected


def test_vmrun_begin_recording():
    """Test begin_recording method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'beginRecording', '/tmp/first/some.vmx', 'a_name']
    got = vmrun.begin_recording('a_name')
    assert got == expected


def test_vmrun_end_recording():
    """Test end_recording method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'endRecording', '/tmp/first/some.vmx']
    got = vmrun.end_recording()
    assert got == expected


def test_vmrun_begin_replay():
    """Test begin_replay method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'beginReplay', '/tmp/first/some.vmx', 'a_name']
    got = vmrun.begin_replay('a_name')
    assert got == expected


def test_vmrun_end_replay():
    """Test end_replay method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'endReplay', '/tmp/first/some.vmx']
    got = vmrun.end_replay()
    assert got == expected


def test_vmrun_vprobe_version():
    """Test vprobe_version method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'vprobeVersion', '/tmp/first/some.vmx']
    got = vmrun.vprobe_version()
    assert got == expected


def test_vmrun_vprobe_load():
    """Test vprobe_load method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'vprobeLoad', '/tmp/first/some.vmx', 'a_script']
    got = vmrun.vprobe_load('a_script')
    assert got == expected


def test_vmrun_vprobe_load_file():
    """Test vprobe_load_file method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'vprobeLoadFile', '/tmp/first/some.vmx', 'a_script']
    got = vmrun.vprobe_load_file('a_script')
    assert got == expected


def test_vmrun_vprobe_reset():
    """Test vprobe_reset method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'vprobeReset', '/tmp/first/some.vmx']
    got = vmrun.vprobe_reset()
    assert got == expected


def test_vmrun_vprobe_list_probes():
    """Test vprobe_list_probes method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'vprobeListProbes', '/tmp/first/some.vmx']
    got = vmrun.vprobe_list_probes()
    assert got == expected


def test_vmrun_vprobe_list_globals():
    """Test vprobe_list_globals method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    expected = ['/tmp/vmrun', '-T', 'ws', 'vprobeListGlobals', '/tmp/first/some.vmx']
    got = vmrun.vprobe_list_globals()
    assert got == expected


@patch('mech.vmrun.VMrun.check_tools_state', return_value='')
def test_vmrun_installed_tools_not_running(mock_tools_state):
    """Test installed_tools method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    assert not vmrun.check_tools_state()


@patch('mech.vmrun.VMrun.check_tools_state', return_value='running')
def test_vmrun_installed_tools_running(mock_tools_state):
    """Test installed_tools method."""
    vmrun = mech.vmrun.VMrun('/tmp/first/some.vmx', executable='/tmp/vmrun',
                             provider='ws', test_mode=True)
    assert vmrun.check_tools_state()



# Copyright (c) 2020 Mike Kinney
"""Tests for VBoxManage class."""
from unittest.mock import patch, MagicMock


import mech.vbm


@patch('subprocess.Popen')
def test_vbm_run(mock_popen):
    """Test run method."""
    process_mock = MagicMock()
    attrs = {'communicate.return_value': ('output', 'error')}
    process_mock.configure_mock(**attrs)
    mock_popen.return_value = process_mock
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=False)
    got = vbm.run('list')
    assert got is None
    mock_popen.assert_called()


@patch('os.path.exists', return_value=True)
def test_vbm_installed(mock_path_exists):
    """Test installed."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage')
    assert vbm.installed()
    mock_path_exists.assert_called()


@patch('os.path.exists', return_value=False)
def test_vbm_executable_not_found(mock_path_exists):
    """Test installed."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage')
    assert not vbm.installed()
    mock_path_exists.assert_called()


def test_vbm_importvm():
    """Test importvm method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'import', '/tmp/first_tmp/some.ovf',
                '--vsys', '0', '--vmname', 'first', '--basefolder', '/tmp/first']
    got = vbm.importvm(path_to_ovf='/tmp/first_tmp/some.ovf',
                       name='first', base_folder='/tmp/first')
    assert got == expected


def test_vbm_start_headless():
    """Test start method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'startvm', 'first', '--type', 'headless']
    got = vbm.start(vmname='first')
    assert got == expected


def test_vbm_start_gui():
    """Test start method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'startvm', 'first', '--type', 'gui']
    got = vbm.start(vmname='first', gui=True)
    assert got == expected


def test_vbm_register():
    """Test register method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'registervm', '/tmp/some.ovf']
    got = vbm.register('/tmp/some.ovf')
    assert got == expected


def test_vbm_unregister():
    """Test unregister method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'unregistervm', 'first']
    got = vbm.unregister('first')
    assert got == expected


def test_vbm_stop():
    """Test stop method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'controlvm', 'first', 'poweroff']
    got = vbm.stop('first')
    assert got == expected


def test_vbm_pause():
    """Test pause method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'controlvm', 'first', 'pause']
    got = vbm.pause('first')
    assert got == expected


def test_vbm_list_hostonly_ifs():
    """Test list_hostonly_if method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'list', 'hostonlyifs']
    got = vbm.list_hostonly_ifs()
    assert got == expected


def test_vbm_create_hostonly_if():
    """Test create_hostonly_if method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'hostonlyif', 'create']
    got = vbm.create_hostonly_if()
    assert got == expected


def test_vbm_remove_hostonly_if():
    """Test remove_hostonly_if method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'hostonlyif', 'remove', 'vboxnet0']
    got = vbm.remove_hostonly_if()
    assert got == expected


def test_vbm_add_hostonly_dhcp():
    """Test add_hostonly_dhcp method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'dhcpserver', 'add', '--ifname', 'vboxnet0',
                '--enable', '--ip', '192.168.56.1', '--netmask', '255.255.255.0',
                '--lower-ip', '192.168.56.100', '--upper-ip', '192.168.56.200']
    got = vbm.add_hostonly_dhcp()
    assert got == expected


def test_vbm_remove_hostonly_dhcp():
    """Test remove_hostonly_dhcp method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'dhcpserver', 'remove', '--network',
                'HostInterfaceNetworking-vboxnet0']
    got = vbm.remove_hostonly_dhcp()
    assert got == expected


def test_vbm_list_dhcpservers():
    """Test list_dhcpservers method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'list', 'dhcpservers']
    got = vbm.list_dhcpservers()
    assert got == expected


def test_vbm_hostonly():
    """Test hostonly method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'modifyvm', 'first', '--nic1', 'hostonly',
                '--hostonlyadapter1', 'vboxnet0']
    got = vbm.hostonly('first')
    assert got == expected


def test_vbm_list():
    """Test list method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'list', 'vms']
    got = vbm.list()
    assert got == expected



# -*- coding: utf-8 -*-
#
# Copyright (c) 2018 German Mendez Bravo (Kronuz)
# Copyright (c) 2020 Mike Kinney
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to
# deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
#
"""VBM (virtual box manage) class"""

from __future__ import absolute_import

import os
import sys
import re
import logging
import subprocess
import time

from .compat import b2s
from . import utils


LOGGER = logging.getLogger(__name__)


class VBoxManage():
    """Interface class for the 'VBoxManage' command.
       The 'VBoxManage' command is used to interact with Oracle Virtual Box instances.
       To add/update functionality, run the 'VBoxManage' command with '--help'.
    """

    def __init__(self, executable=None, test_mode=False):
        """Constructor - set sane defaults."""

        self.executable = executable
        self.provider = 'virtualbox'

        if self.executable is None:
            if sys.platform == 'darwin':
                self.executable = utils.get_darwin_executable('VBoxManage')
            elif sys.platform == 'win32':
                # FUTURE: Find out the windows registry/path info
                self.executable = None
            else:
                self.executable = utils.get_fallback_executable('VBoxManage')

        if self.executable is not None:
            LOGGER.debug('self.executable:%s', self.executable)

        # If test_mode is True, then do not perform the action
        # just return the command info
        self.test_mode = test_mode

    def installed(self):
        """Returns True if VB is installed (based on whether
           we could find the VBoxManage command."""
        if self.executable is not None and os.path.exists(self.executable):
            return True
        else:
            return False

    def run(self, cmd, *args, **kwargs):
        """Execute a command."""
        quiet = kwargs.pop('quiet', False)
        arguments = kwargs.pop('arguments', ())

        cmds = [self.executable]
        cmds.append(cmd)
        cmds.extend(filter(None, args))
        cmds.extend(filter(None, arguments))

        LOGGER.debug(
            " ".join(
                "'{}'".format(
                    c.replace(
                        "'",
                        "\\'")) if ' ' in c else c for c in cmds))

        if self.test_mode:
            return cmds

        startupinfo = None
        if os.name == "nt":
            startupinfo = subprocess.STARTUPINFO()
            startupinfo.dwFlags |= subprocess.SW_HIDE | subprocess.STARTF_USESHOWWINDOW
        proc = subprocess.Popen(
            cmds,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            startupinfo=startupinfo)
        stdoutdata, stderrdata = map(b2s, proc.communicate())

        if stderrdata and not quiet:
            LOGGER.error(stderrdata.strip())
        LOGGER.debug("(⏎ %s)", proc.returncode)

        if not proc.returncode:
            stdoutdata = stdoutdata.strip()
            LOGGER.debug(repr(stdoutdata))
            return stdoutdata

        if stdoutdata and not quiet:
            LOGGER.error(stdoutdata.strip())

    ############################################################################
    # startvm               <uuid|vmname>...
    #                       [--type gui|headless|separate]
    # registervm            <filename>
    # unregistervm          <uuid|vmname> [--delete]
    # import                <ovfname/ovaname>
    #                       [--dry-run|-n]
    #                       [--options keepallmacs|keepnatmacs|importtovdi]
    #                       [--vmname <name>]
    #                       [--cloud]
    #                       [--cloudprofile <cloud profile name>]
    #                       [--cloudinstanceid <instance id>]
    #                       [--cloudbucket <bucket name>]
    #                       [more options]
    #                       (run with -n to have options displayed
    #                        for a particular OVF. It doesn't work for the Cloud import.)

    def start(self, vmname, gui=False, quiet=False):
        '''Start a VM'''
        return self.run('startvm', vmname, '--type', 'gui' if gui else 'headless', quiet=quiet)

    def importvm(self, path_to_ovf, name, base_folder, quiet=False):
        '''Import a VM (import ovf/ova).
           Note: Did not want to use 'import' as that could clash with python's "import".
           Tip: Do not have spaces in the options, make them be a separate argument.
                (ex: For the argument "--vsys 0" option, use: '--vsys', '0')
        '''
        return self.run('import', path_to_ovf, '--vsys', '0', '--vmname', name,
                        '--basefolder', base_folder, quiet=quiet)

    def _ip(self, vmname, quiet=False):
        """Get ip address of VM."""
        line = self.run('guestproperty', 'get', vmname,
                        '/VirtualBox/GuestInfo/Net/0/V4/IP', quiet=False)
        if line and line != 'No value set!':
            parts = line.split()
            if len(parts) > 1:
                return parts[1]

    def ip(self, vmname, wait=False, quiet=False):
        """Get ip address of VM.
        """
        ip_address = self._ip(vmname, quiet=quiet)
        if wait:
            while True:
                ip_address = self._ip(vmname, quiet=quiet)
                if ip_address:
                    return ip_address
                time.sleep(1)
        else:
            return ip_address

    def register(self, filename, quiet=False):
        '''Register a VM.
           Note: Probably want to use importvm().
        '''
        return self.run('registervm', filename, quiet=quiet)

    def unregister(self, vmname, quiet=False):
        '''Unregister a VM (similar to destroy)'''
        return self.run('unregistervm', vmname, quiet=quiet)

    # controlvm                 <uuid|vmname>
    #                       pause|resume|reset|poweroff|savestate|
    #                          ...snip...

    def stop(self, vmname, quiet=False):
        '''Stop a VM'''
        return self.run('controlvm', vmname, 'poweroff', quiet=quiet)

    def pause(self, vmname, quiet=False):
        '''Pause a VM'''
        return self.run('controlvm', vmname, 'pause', quiet=quiet)

    def list_hostonly_ifs(self, quiet=False):
        '''List hostonly interfaces.'''
        return self.run('list', 'hostonlyifs', quiet=quiet)

    def create_hostonly(self, quiet=False):
        '''Create the stuff needed for hostonly networking to work.'''
        ifs = self.list_hostonly_ifs()
        if re.search(r'vboxnet', ifs, re.MULTILINE) is None:
            self.create_hostonly_if(quiet=quiet)
        ds = self.list_dhcpservers(quiet=quiet)
        if re.search(r'vboxnet', ds, re.MULTILINE) is None:
            self.add_hostonly_dhcp(quiet=quiet)

    def create_hostonly_if(self, quiet=False):
        '''Create hostonly interface (creates vboxnet0)'''
        return self.run('hostonlyif', 'create', quiet=quiet)

    def remove_hostonly_if(self, host_interface='vboxnet0', quiet=False):
        '''Remove hostonly interface (creates vboxnet0)'''
        return self.run('hostonlyif', 'remove', host_interface, quiet=quiet)

    def add_hostonly_dhcp(self, host_interface='vboxnet0', quiet=False):
        '''Create dhcp on host interface (ex: vboxnet0).'''
        return self.run('dhcpserver', 'add', '--ifname', host_interface,
                        '--enable', '--ip', '192.168.56.1', '--netmask', '255.255.255.0',
                        '--lower-ip', '192.168.56.100', '--upper-ip', '192.168.56.200',
                        quiet=quiet)

    def remove_hostonly_dhcp(self, network_name='HostInterfaceNetworking-vboxnet0', quiet=False):
        '''Remove dhcp network.'''
        return self.run('dhcpserver', 'remove', '--network', network_name, quiet=quiet)

    def list_dhcpservers(self, quiet=False):
        '''List dhcpservers.'''
        return self.run('list', 'dhcpservers', quiet=quiet)

    def hostonly(self, vmname, quiet=False):
        '''Make a VM use hostonly networking'''
        return self.run('modifyvm', vmname, '--nic1', 'hostonly',
                        '--hostonlyadapter1', 'vboxnet0', quiet=quiet)

    ############################################################################
    # list [--long|-l] [--sorted|-s]          vms|runningvms|ostypes|hostdvds|hostfloppies|
    #                       intnets|bridgedifs|hostonlyifs|natnets|dhcpservers|
    #                       hostinfo|hostcpuids|hddbackends|hdds|dvds|floppies|
    #                       usbhost|usbfilters|systemproperties|extpacks|
    #                       groups|webcams|screenshotformats|cloudproviders|
    #                       cloudprofiles

    def list(self, quiet=False):
        '''List all VMs'''
        return self.run('list', 'vms', quiet=quiet)

    def list_running(self, quiet=False):
        '''List all running VMs'''
        running_vms = []
        output = self.run('list', 'runningvms', quiet=quiet)
        each_line = output.split('\n')
        LOGGER.debug('each_line:%s', each_line)
        for line in each_line:
            LOGGER.debug('line:%s', line)
            parts = line.split(' ')
            if len(parts) > 0:
                running_vms.append(parts[0].replace('"', ''))
        LOGGER.debug('running_vms:%s', running_vms)
        return running_vms



# -*- coding: utf-8 -*-
#
# Copyright (c) 2018 German Mendez Bravo (Kronuz)
# Copyright (c) 2020 Mike Kinney
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to
# deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
#
"""VMrun class"""

from __future__ import absolute_import

import os
import sys
import logging
import subprocess
import tempfile

from .compat import b2s
from . import utils


LOGGER = logging.getLogger(__name__)


class VMrun():  # pylint: disable=too-many-public-methods
    """Interface class for the 'vmrun' command.
       The 'vmrun' command is used to interact with VMware.
       To add/update vmware functionality, run the 'vmrun' command with '--help'.
    """

    def __init__(self, vmx_file=None,  # pylint: disable=too-many-arguments
                 user=None, password=None, executable=None, provider=None,
                 test_mode=False):
        """Constructor - set sane defaults."""
        self.vmx_file = vmx_file
        self.user = user
        self.password = password
        self.executable = executable
        self.provider = provider

        if self.executable is None:
            if sys.platform == 'darwin':
                self.executable = utils.get_darwin_executable()
            elif sys.platform == 'win32':
                self.executable = utils.get_win32_executable()
            else:
                self.executable = utils.get_fallback_executable()
        if self.provider is None:
            if self.executable is not None:
                self.provider = utils.get_provider(self.executable)
                LOGGER.debug('self.executable:%s self.provider:%s',
                             self.executable, self.provider)
        # If test_mode is True, then do not perform the action
        # just return the command info
        self.test_mode = test_mode

    def installed(self):
        """Returns True if vmware is installed (based on whether we
           could find the vmrun command."""
        if self.executable is not None and os.path.exists(self.executable):
            return True
        else:
            return False

    def vmrun(self, cmd, *args, **kwargs):
        """Execute a 'vmrun' command."""
        quiet = kwargs.pop('quiet', False)
        arguments = kwargs.pop('arguments', ())

        cmds = [self.executable]
        cmds.append('-T')
        cmds.append(self.provider)
        if self.user:
            cmds.append('-gu')
            cmds.append(self.user)
        if self.password:
            cmds.append('-gp')
            cmds.append(self.password)
        cmds.append(cmd)
        cmds.extend(filter(None, args))
        cmds.extend(filter(None, arguments))

        LOGGER.debug(
            " ".join(
                "'{}'".format(
                    c.replace(
                        "'",
                        "\\'")) if ' ' in c else c for c in cmds))

        if self.test_mode:
            return cmds

        startupinfo = None
        if os.name == "nt":
            startupinfo = subprocess.STARTUPINFO()
            startupinfo.dwFlags |= subprocess.SW_HIDE | subprocess.STARTF_USESHOWWINDOW
        proc = subprocess.Popen(
            cmds,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            startupinfo=startupinfo)
        stdoutdata, stderrdata = map(b2s, proc.communicate())

        if stderrdata and not quiet:
            LOGGER.error(stderrdata.strip())
        LOGGER.debug("(⏎ %s)", proc.returncode)

        if not proc.returncode:
            stdoutdata = stdoutdata.strip()
            LOGGER.debug(repr(stdoutdata))
            return stdoutdata

        if stdoutdata and not quiet:
            LOGGER.error(stdoutdata.strip())

    ############################################################################
    # POWER COMMANDS           PARAMETERS           DESCRIPTION
    # --------------           ----------           -----------
    # start                    Path to vmx file     Start a VM or Team
    #                          [gui|nogui]
    #
    # stop                     Path to vmx file     Stop a VM or Team
    #                          [hard|soft]
    #
    # reset                    Path to vmx file     Reset a VM or Team
    #                          [hard|soft]
    #
    # suspend                  Path to vmx file     Suspend a VM or Team
    #                          [hard|soft]
    #
    # pause                    Path to vmx file     Pause a VM
    #
    # unpause                  Path to vmx file     Unpause a VM
    #

    def start(self, gui=False, quiet=False):
        '''Start a VM or Team'''
        return self.vmrun('start', self.vmx_file, 'gui' if gui else 'nogui', quiet=quiet)

    def stop(self, mode='soft', quiet=False):
        '''Stop a VM or Team'''
        return self.vmrun('stop', self.vmx_file, mode, quiet=quiet)

    def reset(self, mode='soft', quiet=False):
        '''Reset a VM or Team'''
        return self.vmrun('reset', self.vmx_file, mode, quiet=quiet)

    def suspend(self, mode='soft', quiet=False):
        '''Suspend a VM or Team'''
        return self.vmrun('suspend', self.vmx_file, mode, quiet=quiet)

    def pause(self, quiet=False):
        '''Pause a VM'''
        return self.vmrun('pause', self.vmx_file, quiet=quiet)

    def unpause(self, quiet=False):
        '''Unpause a VM'''
        return self.vmrun('unpause', self.vmx_file, quiet=quiet)

    ############################################################################
    # SNAPSHOT COMMANDS        PARAMETERS           DESCRIPTION
    # -----------------        ----------           -----------
    # listSnapshots            Path to vmx file     List all snapshots in a VM
    #                          [showTree]
    #
    # snapshot                 Path to vmx file     Create a snapshot of a VM
    #                          Snapshot name
    #
    # deleteSnapshot           Path to vmx file     Remove a snapshot from a VM
    #                          Snapshot name
    #                          [andDeleteChildren]
    #
    # revertToSnapshot         Path to vmx file     Set VM state to a snapshot
    #                          Snapshot name
    #

    def list_snapshots(self, show_tree=False, quiet=False):
        '''List all snapshots in a VM'''
        return self.vmrun(
            'listSnapshots',
            self.vmx_file,
            'showTree' if show_tree else None,
            quiet=quiet)

    def snapshot(self, snap_name, quiet=False):
        '''Create a snapshot of a VM'''
        return self.vmrun('snapshot', self.vmx_file, snap_name, quiet=quiet)

    def delete_snapshot(self, snap_name, and_delete_children=False, quiet=False):
        '''Remove a snapshot from a VM'''
        return self.vmrun(
            'deleteSnapshot',
            self.vmx_file,
            snap_name,
            'andDeleteChildren' if and_delete_children else None,
            quiet=quiet)

    def revert_to_snapshot(self, snap_name, quiet=False):
        '''Set VM state to a snapshot'''
        return self.vmrun('revertToSnapshot', self.vmx_file, snap_name, quiet=quiet)

    ############################################################################
    # NETWORKADAPTER COMMANDS  PARAMETERS           DESCRIPTION
    # -----------------------  ----------           -----------
    # listNetworkAdapters      Path to vmx file     List all network adapters in a VM
    #
    #
    # addNetworkAdapter        Path to vmx file     Add a network adapter on a VM
    #                          Network adapter type
    #                          [Host nework]
    #
    #
    # setNetworkAdapter        Path to vmx file     Update a network adapter on a VM
    #                          Network adapter index
    #                          Network adapter type
    #                          [Host network]
    #
    #
    # deleteNetworkAdapter     Path to vmx file     Remove a network adapter on a VM
    #                          Network adapter index

    def list_network_adapters(self, quiet=False):
        '''List all network adapters in a VM'''
        return self.vmrun('listNetworkAdapters', self.vmx_file, quiet=quiet)

    def add_network_adapter(self, adapter_type, host_network=None, quiet=False):
        '''Add a network adapter on a VM'''
        return self.vmrun(
            'addNetworkAdapter',
            self.vmx_file,
            adapter_type,
            host_network,
            quiet=quiet)

    def set_network_adapter(self, adapter_index, adapter_type, host_network=None, quiet=False):
        '''Update a network adapter on a VM'''
        return self.vmrun(
            'setNetworkAdapter',
            self.vmx_file,
            adapter_index,
            adapter_type,
            host_network,
            quiet=quiet)

    def delete_network_adapter(self, adapter_index, quiet=False):
        '''Remove a network adapter on a VM'''
        return self.vmrun('deleteNetworkAdapter', self.vmx_file, adapter_index, quiet=quiet)

    ############################################################################
    # HOST NETWORK COMMANDS    PARAMETERS           DESCRIPTION
    # ---------------------    ----------           -----------
    # listHostNetworks                              List all networks in the host
    #
    # listPortForwardings      Host network name    List all available port
    #                                               forwardings on a host network
    #
    #
    # setPortForwarding        Host network name    Add or update a port
    #                                               forwarding on a host network
    #                          Protocol
    #                          Host port
    #                          Guest ip
    #                          Guest port
    #                          [Description]
    #
    # deletePortForwarding     Host network name    Delete a port forwarding on a host network
    #                          Protocol
    #                          Host port

    def list_host_networks(self, quiet=False):
        '''List all networks in the host.
           Note: Not available on linux.
        '''
        return self.vmrun('listHostNetworks', quiet=quiet)

    def list_port_forwardings(self, host_network, quiet=False):
        '''List all available port forwardings on a host network'''
        return self.vmrun('listPortForwardings', host_network, quiet=quiet)

    def set_port_forwarding(  # pylint: disable=too-many-arguments
            self,
            host_network,
            protocol,
            host_port,
            guest_ip,
            guest_port,
            description=None,
            quiet=False):
        '''Add or update a port forwarding on a host network'''
        return self.vmrun(
            'setPortForwarding',
            host_network,
            protocol,
            host_port,
            guest_ip,
            guest_port,
            description,
            quiet=quiet)

    def delete_port_forwarding(self, host_network, protocol, host_port,
                               quiet=False):  # pylint: disable=too-many-arguments
        '''Delete a port forwarding on a host network'''
        return self.vmrun('deletePortForwarding', host_network, protocol, host_port, quiet=quiet)

    ############################################################################
    # GUEST OS COMMANDS        PARAMETERS           DESCRIPTION
    # -----------------        ----------           -----------
    # runProgramInGuest        Path to vmx file     Run a program in Guest OS
    #                          [-noWait]
    #                          [-activeWindow]
    #                          [-interactive]
    #                          Complete-Path-To-Program
    #                          [Program arguments]
    #
    # fileExistsInGuest        Path to vmx file     Check if a file exists in Guest OS
    #                          Path to file in guest
    #
    # directoryExistsInGuest   Path to vmx file     Check if a directory exists in Guest OS
    #                          Path to directory in guest
    #
    # setSharedFolderState     Path to vmx file     Modify a Host-Guest shared folder
    #                          Share name
    #                          Host path
    #                          writable | readonly
    #
    # addSharedFolder          Path to vmx file     Add a Host-Guest shared folder
    #                          Share name
    #                          New host path
    #
    # removeSharedFolder       Path to vmx file     Remove a Host-Guest shared folder
    #                          Share name
    #
    # enableSharedFolders      Path to vmx file     Enable shared folders in Guest
    #                          [runtime]
    #
    # disableSharedFolders     Path to vmx file     Disable shared folders in Guest
    #                          [runtime]
    #
    # listProcessesInGuest     Path to vmx file     List running processes in Guest OS
    #
    # killProcessInGuest       Path to vmx file     Kill a process in Guest OS
    #                          process id
    #
    # runScriptInGuest         Path to vmx file     Run a script in Guest OS
    #                          [-noWait]
    #                          [-activeWindow]
    #                          [-interactive]
    #                          Interpreter path
    #                          Script text
    #
    # deleteFileInGuest        Path to vmx file     Delete a file in Guest OS
    #                          Path in guest
    #
    # createDirectoryInGuest   Path to vmx file     Create a directory in Guest OS
    #                          Directory path in guest
    #
    # deleteDirectoryInGuest   Path to vmx file     Delete a directory in Guest OS
    #                          Directory path in guest
    #
    # createTempfileInGuest    Path to vmx file     Create a temporary file in Guest OS
    #
    # listDirectoryInGuest     Path to vmx file     List a directory in Guest OS
    #                          Directory path in guest
    #
    # copyFileFromHostToGuest  Path to vmx file     Copy a file from host OS to guest OS
    #                          Path on host
    #                          Path in guest
    #
    # copyFileFromGuestToHost  Path to vmx file     Copy a file from guest OS to host OS
    #                          Path in guest
    #                          Path on host
    #
    # renameFileInGuest        Path to vmx file     Rename a file in Guest OS
    #                          Original name
    #                          New name
    #
    # typeKeystrokesInGuest    Path to vmx file     Type Keystrokes in Guest OS
    #                          keystroke string
    #
    # connectNamedDevice       Path to vmx file     Connect the named device in the Guest OS
    #                          device name
    #
    # disconnectNamedDevice    Path to vmx file     Disconnect the named device in the Guest OS
    #                          device name
    #
    # captureScreen            Path to vmx file     Capture the screen of the VM to a local file
    #                          Path on host
    #
    # writeVariable            Path to vmx file     Write a variable in the VM state
    #                          [runtimeConfig|guestEnv|guestVar]
    #                          variable name
    #                          variable value
    #
    # readVariable             Path to vmx file     Read a variable in the VM state
    #                          [runtimeConfig|guestEnv|guestVar]
    #                          variable name
    #
    # getGuestIPAddress        Path to vmx file     Gets the IP address of the guest
    #                          [-wait]
    #

    def run_program_in_guest(  # pylint: disable=too-many-arguments
            self,
            program_path,
            program_arguments=None,
            wait=True,
            activate_window=False,
            interactive=False,
            quiet=False):
        """Run a program in the guest vm."""
        if program_arguments is None:
            program_arguments = []
        return self.vmrun(
            'runProgramInGuest',
            self.vmx_file,
            None if wait else '-noWait',
            '-activateWindow' if activate_window else None,
            '-interactive' if interactive else None,
            program_path,
            arguments=program_arguments,
            quiet=quiet)

    def set_shared_folder_state(self, share_name, new_path, mode='readonly', quiet=False):
        '''Modify a Host-Guest shared folder'''
        return self.vmrun(
            'setSharedFolderState',
            self.vmx_file,
            share_name,
            new_path,
            mode,
            quiet=quiet)

    def add_shared_folder(self, share_name, host_path, quiet=False):
        '''Add a Host-Guest shared folder'''
        return self.vmrun('addSharedFolder', self.vmx_file, share_name, host_path, quiet=quiet)

    def remove_shared_folder(self, share_name, quiet=False):
        '''Remove a Host-Guest shared folder'''
        return self.vmrun('removeSharedFolder', self.vmx_file, share_name, quiet=quiet)

    def enable_shared_folders(self, runtime=None, quiet=False):
        '''Enable shared folders.'''
        return self.vmrun('enableSharedFolders', self.vmx_file, runtime, quiet=quiet)

    def disable_shared_folders(self, runtime=None, quiet=False):
        '''Disable shared folders in Guest'''
        return self.vmrun('disableSharedFolders', self.vmx_file, runtime, quiet=quiet)

    def list_processes_in_guest(self, quiet=False):
        '''List running processes in Guest OS'''
        return self.vmrun('listProcessesInGuest', self.vmx_file, quiet=quiet)

    def kill_process_in_guest(self, pid, quiet=False):
        '''Kill a process in Guest OS'''
        return self.vmrun('killProcessInGuest', self.vmx_file, pid, quiet=quiet)

    def run_script_in_guest(  # pylint: disable=too-many-arguments
            self,
            interpreter_path,
            script,
            wait=True,
            activate_window=False,
            interactive=False,
            quiet=False):
        '''Run a script in Guest OS'''
        return self.vmrun(
            'runScriptInGuest',
            self.vmx_file,
            interpreter_path,
            script,
            None if wait else '-noWait',
            '-activateWindow' if activate_window else None,
            '-interactive' if interactive else None,
            quiet=quiet)

    def delete_file_in_guest(self, filename, quiet=False):
        '''Delete a file in Guest OS'''
        return self.vmrun('deleteFileInGuest', self.vmx_file, filename, quiet=quiet)

    def create_directory_in_guest(self, path, quiet=False):
        '''Create a directory in Guest OS'''
        return self.vmrun('createDirectoryInGuest', self.vmx_file, path, quiet=quiet)

    def delete_directory_in_guest(self, path, quiet=False):
        '''Delete a directory in Guest OS'''
        return self.vmrun('deleteDirectoryInGuest', self.vmx_file, path, quiet=quiet)

    def create_tempfile_in_guest(self, quiet=False):
        '''Create a temporary file in Guest OS'''
        return self.vmrun('createTempfileInGuest', self.vmx_file, quiet=quiet)

    def list_directory_in_guest(self, path, quiet=False):
        '''List a directory in Guest OS'''
        return self.vmrun('listDirectoryInGuest', self.vmx_file, path, quiet=quiet)

    def copy_file_from_host_to_guest(self, host_path, guest_path, quiet=False):
        '''Copy a file from host OS to guest OS'''
        return self.vmrun(
            'copyFileFromHostToGuest',
            self.vmx_file,
            host_path,
            guest_path,
            quiet=quiet)

    def copy_file_from_guest_to_host(self, guest_path, host_path, quiet=False):
        '''Copy a file from guest OS to host OS'''
        return self.vmrun(
            'copyFileFromGuestToHost',
            self.vmx_file,
            guest_path,
            host_path,
            quiet=quiet)

    def rename_file_in_guest(self, original_name, new_name, quiet=False):
        '''Rename a file in Guest OS'''
        return self.vmrun('renameFileInGuest', self.vmx_file, original_name, new_name, quiet=quiet)

    def type_keystrokes_in_guest(self, keystroke, quiet=False):
        '''Type Keystrokes in Guest OS'''
        return self.vmrun('typeKeystrokesInGuest', self.vmx_file, keystroke, quiet=quiet)

    def connect_named_device(self, device_name, quiet=False):
        '''Connect the named device in the Guest OS'''
        return self.vmrun('connectNamedDevice', self.vmx_file, device_name, quiet=quiet)

    def disconnect_named_device(self, device_name, quiet=False):
        '''Disconnect the named device in the Guest OS'''
        return self.vmrun('disconnectNamedDevice', self.vmx_file, device_name, quiet=quiet)

    def capture_screen(self, path_on_host, quiet=False):
        '''Capture the screen of the VM to a local file'''
        return self.vmrun('captureScreen', self.vmx_file, path_on_host, quiet=quiet)

    def write_variable(self, var_name, var_value, mode=None, quiet=False):
        '''Write a variable in the VM state'''
        return self.vmrun('writeVariable', self.vmx_file, mode, var_name, var_value, quiet=quiet)

    def read_variable(self, var_name, mode=None, quiet=False):
        '''Read a variable in the VM state'''
        return self.vmrun('readVariable', self.vmx_file, mode, var_name, quiet=quiet)

    def get_guest_ip_address(self, wait=True, quiet=False, lookup=False):
        '''Gets the IP address of the guest'''
        if lookup is True:
            guest_tmp_filename = '/tmp/.ip_address'
            cmd = "ifconfig | grep -Eo 'inet (addr:)?([0-9]*\\.){3}[0-9]*'"
            cmd += "| grep -Eo '([0-9]*\\.){3}[0-9]*' | grep -v '127.0.0.1' > " + guest_tmp_filename
            LOGGER.debug("cmd:%s", cmd)
            self.run_script_in_guest('/bin/sh', cmd, quiet=quiet)
            temp_file = tempfile.NamedTemporaryFile(delete=False)
            try:
                temp_file.close()
                self.copy_file_from_guest_to_host(guest_tmp_filename, temp_file.name, quiet=quiet)
                ip_addresses = open(temp_file.name).read().split()
                # clean up the guest tmp file
                self.run_script_in_guest(
                    '/bin/sh',
                    "rm {}".format(guest_tmp_filename),
                    quiet=False)
                if ip_addresses:
                    return ip_addresses[0]
                else:
                    return None
            finally:
                os.unlink(temp_file.name)
        else:
            ip_address = self.vmrun('getGuestIPAddress', self.vmx_file,
                                    '-wait' if wait else None, quiet=quiet)
            if ip_address == 'unknown':
                ip_address = ''
        return ip_address

    ############################################################################
    # GENERAL COMMANDS         PARAMETERS           DESCRIPTION
    # ----------------         ----------           -----------
    # list                                          List all running VMs
    #
    # upgradevm                Path to vmx file     Upgrade VM file format, virtual hw
    #
    # installTools             Path to vmx file     Install Tools in Guest
    #
    # checkToolsState          Path to vmx file     Check the current Tools state
    #
    # deleteVM                 Path to vmx file     Delete a VM
    #
    # clone                    Path to vmx file     Create a copy of the VM
    #                          Path to destination vmx file
    #                          full|linked
    #                          [-snapshot=Snapshot Name]
    #                          [-cloneName=Name]

    def list(self, quiet=False):
        '''List all running VMs'''
        return self.vmrun('list', self.vmx_file, quiet=quiet)

    def upgradevm(self, quiet=False):
        '''Upgrade VM file format, virtual hw.
           Note: The vm must be stopped before running this command.
        '''
        return self.vmrun('upgradevm', self.vmx_file, quiet=quiet)

    def install_tools(self, quiet=False):
        '''Install Tools in Guest OS'''
        return self.vmrun('installTools', self.vmx_file, quiet=quiet)

    def check_tools_state(self, quiet=False):
        '''Check the current Tools state'''
        return self.vmrun('checkToolsState', self.vmx_file, quiet=quiet)

    def register(self, quiet=False):
        # unavailable in VMware Fusion 10 (OS X)?
        '''Register a VM'''
        return self.vmrun('register', self.vmx_file, quiet=quiet)

    def unregister(self, quiet=False):
        # unavailable in VMware Fusion 10 (OS X)?
        '''Unregister a VM'''
        return self.vmrun('unregister', self.vmx_file, quiet=quiet)

    def list_registered_vm(self, quiet=False):
        # unavailable in VMware Fusion 10 (OS X)?
        '''List registered VMs'''
        return self.vmrun('listRegisteredVM', self.vmx_file, quiet=quiet)

    def delete_vm(self, quiet=False):
        '''Delete a VM'''
        return self.vmrun('deleteVM', self.vmx_file, quiet=quiet)

    def clone(self, dest_vmx, mode, snap_name=None, quiet=False):
        '''Create a copy of the VM'''
        return self.vmrun('clone', self.vmx_file, dest_vmx, mode, snap_name, quiet=quiet)

    ############################################################################
    # RECORD/REPLAY COMMANDS   PARAMETERS           DESCRIPTION
    # ----------------------   ----------           -----------
    # beginRecording           Path to vmx file     Begin recording a VM
    #                          Snapshot name
    #
    # endRecording             Path to vmx file     End recording a VM
    #
    # beginReplay              Path to vmx file     Begin replaying a VM
    #                          Snapshot name
    #
    # endReplay                Path to vmx file     End replaying a VM

    def begin_recording(self, snap_name, quiet=False):
        # unavailable in VMware Fusion 10 (OS X)?
        '''Begin recording a VM'''
        return self.vmrun('beginRecording', self.vmx_file, snap_name, quiet=quiet)

    def end_recording(self, quiet=False):
        # unavailable in VMware Fusion 10 (OS X)?
        '''End recording a VM'''
        return self.vmrun('endRecording', self.vmx_file, quiet=quiet)

    def begin_replay(self, snap_name, quiet=False):
        # unavailable in VMware Fusion 10 (OS X)?
        '''Begin replaying a VM'''
        return self.vmrun('beginReplay', self.vmx_file, snap_name, quiet=quiet)

    def end_replay(self, quiet=False):
        # unavailable in VMware Fusion 10 (OS X)?
        '''End replaying a VM'''
        return self.vmrun('endReplay', self.vmx_file, quiet=quiet)

    ############################################################################
    # VPROBE COMMANDS          PARAMETERS           DESCRIPTION
    # ---------------          ----------           -----------
    # vprobeVersion            Path to vmx file     List VP version
    #
    # vprobeLoad               Path to vmx file     Load VP script
    #                          'VP script text'
    #
    # vprobeLoadFile           Path to vmx file     Load VP file
    #                          Path to VP file
    #
    # vprobeReset              Path to vmx file     Disable all vprobes
    #
    # vprobeListProbes         Path to vmx file     List probes
    #
    # vprobeListGlobals        Path to vmx file     List global variables

    def vprobe_version(self, quiet=False):
        # unavailable in VMware Fusion 10 (OS X)?
        '''List VP version'''
        return self.vmrun('vprobeVersion', self.vmx_file, quiet=quiet)

    def vprobe_load(self, script, quiet=False):
        # unavailable in VMware Fusion 10 (OS X)?
        '''Load VP script'''
        return self.vmrun('vprobeLoad', self.vmx_file, script, quiet=quiet)

    def vprobe_load_file(self, vprobe, quiet=False):
        # unavailable in VMware Fusion 10 (OS X)?
        '''Load VP file'''
        return self.vmrun('vprobeLoadFile', self.vmx_file, vprobe, quiet=quiet)

    def vprobe_reset(self, quiet=False):
        # unavailable in VMware Fusion 10 (OS X)?
        '''Disable all vprobes'''
        return self.vmrun('vprobeReset', self.vmx_file, quiet=quiet)

    def vprobe_list_probes(self, quiet=False):
        # unavailable in VMware Fusion 10 (OS X)?
        '''List probes'''
        return self.vmrun('vprobeListProbes', self.vmx_file, quiet=quiet)

    def vprobe_list_globals(self, quiet=False):
        # unavailable in VMware Fusion 10 (OS X)?
        '''List global variables'''
        return self.vmrun('vprobeListGlobals', self.vmx_file, quiet=quiet)

    ############################################################################

    def installed_tools(self, quiet=False):
        '''Return True if VMware tools is in either 'installed' or 'running',
           otherwise, return False.
        '''
        state = self.check_tools_state(quiet=quiet)
        return state in ('installed', 'running')



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016-2017 Kevin Chung
# Copyright (c) 2018 German Mendez Bravo (Kronuz)
# Copyright (c) 2020 Mike Kinney
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to
# deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
#
"""MechCommand class"""

from __future__ import print_function, absolute_import

import logging

from . import utils
from .command import Command

LOGGER = logging.getLogger(__name__)


class MechCommand(Command):
    """Class for the mech commands from help doc (as python object)."""
    mechfile = None

    def activate_mechfile(self):
        """Load the Mechfile."""
        self.mechfile = utils.load_mechfile()
        LOGGER.debug("loaded mechfile:%s", self.mechfile)

    def instances(self):
        """Returns a list of the instances from the Mechfile."""
        if not self.mechfile:
            self.activate_mechfile()
        return list(self.mechfile)
